package org.whispercomm.manes.apps.helloworld;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.lang.Runnable;

import android.app.Activity;
import android.os.Bundle;
import android.os.RemoteException;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.Toast;

import org.whispercomm.manes.client.maclib.ManesInterface;

/**
 * A Hello World sample application for MANES.
 * <p>
 * This application broadcasts and receives simple "Hello, World!" messages.
 */
public class ManesHelloWorld extends Activity implements Runnable {
	private static final String TAG = "ManesHelloWorld";

	private static final int APP_ID = 253; // 253 is available for
											// experimentation

	private ManesInterface manes;

	private Thread receiveThread;
	private volatile boolean isRunning;

	/**
	 * Called when the activity is first created.
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		LinearLayout layout = new LinearLayout(this);
		layout.setOrientation(LinearLayout.VERTICAL);
		setContentView(layout);

		// Add button to screen for sending messages
		Button button = new Button(this);
		button.setText("Send \"Hello, World!\"");
		layout.addView(button);

		// Configure button to send message when clicked
		button.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				ManesHelloWorld.this.sendMessage();
			}
		});

		// Register with MANES
		try {
			manes = new ManesInterface(APP_ID, this);
		} catch (RemoteException e) {
			this.finish();
			return;
		}

		// Start receive thread
		receiveThread = new Thread(this);
		isRunning = true;
		receiveThread.start();
	}

	/**
	 * Called when the activity is exiting.
	 */
	@Override
	protected void onDestroy() {
		isRunning = false; // Stop receive thread;
		if (receiveThread != null)
			receiveThread.interrupt();
		if (manes != null)
			manes.unregister();
		super.onDestroy();
	}

	/**
	 * Broadcasts a message to all one-hop neighbors.
	 */
	public void sendMessage() {
		try {
			byte[] data = "Hello, World!".getBytes("UTF-8");
			manes.send(data);
		} catch (UnsupportedEncodingException e) {
			Log.e(TAG, "Failed to encode message. Unsupported charset.", e);
		} catch (IOException e) {
			Log.e(TAG, "Error while sending message.", e);
		}
	}

	/**
	 * Receives and displays incoming messages.
	 */
	@Override
	public void run() {
		while (isRunning) {
			byte[] data = null;
			try {
				data = manes.receive(500);
			} catch (IOException e1) {
				Log.e(TAG, e1.getMessage());
				continue;
			} // Timeout after 500 ms, returning null.
			if (null != data) {
				// Debug
				Log.v(TAG, "Received a new packet!");
				try {
					final String msg = new String(data, "UTF-8");
					Log.v(TAG, "Received a packet: " + msg);
					this.runOnUiThread(new Runnable() {
						@Override
						public void run() {
							Log.v(TAG, "Calling Toast.makeText().");
							Toast.makeText(getApplicationContext(),
									String.format("Received: %s", msg),
									Toast.LENGTH_SHORT).show();
						}
					});
				} catch (UnsupportedEncodingException e) {
					Log.e(TAG,
							"Could not decode message. Unsupported charset.", e);
				} catch (RuntimeException e) {
					Log.e(TAG, "Failed to decode message.", e);
				}
			}
		}
	}

}